@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2021 Douglas Davis
+// Copyright (c) 2025 Douglas Davis
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -85,45 +85,10 @@ inline void arr_sqrt(py::array_t<T>& arr) {
   }
 }
 
-/// Threshold for parallel loops for fixed width histograms in 1D.
-inline py::ssize_t fwpt1d() {
-  return py::module_::import("pygram11")
-      .attr("FIXED_WIDTH_PARALLEL_THRESHOLD_1D")
-      .cast<py::ssize_t>();
-}
-
-/// Threshold for parallel loops for fixed width histograms in 2D.
-inline py::ssize_t fwpt2d() {
-  return py::module_::import("pygram11")
-      .attr("FIXED_WIDTH_PARALLEL_THRESHOLD_2D")
-      .cast<py::ssize_t>();
-}
-
-/// Threshold for parallel loops for multiweight fixed width histograms in 1D.
-inline py::ssize_t fwmwpt1d() {
-  return py::module_::import("pygram11")
-      .attr("FIXED_WIDTH_MW_PARALLEL_THRESHOLD_1D")
-      .cast<py::ssize_t>();
-}
-
-/// Threshold for parallel loops for variable width histograms in 1D.
-inline py::ssize_t vwpt1d() {
-  return py::module_::import("pygram11")
-      .attr("VARIABLE_WIDTH_PARALLEL_THRESHOLD_1D")
-      .cast<py::ssize_t>();
-}
-
-/// Threshold for parallel loops for variable width histograms in 2D.
-inline py::ssize_t vwpt2d() {
-  return py::module_::import("pygram11")
-      .attr("VARIABLE_WIDTH_PARALLEL_THRESHOLD_2D")
-      .cast<py::ssize_t>();
-}
-
-/// Threshold for parallel loops for multiweight variable width histograms in 1D.
-inline py::ssize_t vwmwpt1d() {
-  return py::module_::import("pygram11")
-      .attr("VARIABLE_WIDTH_MW_PARALLEL_THRESHOLD_1D")
+inline py::ssize_t config_threshold(const char* k) {
+  return py::module_::import("pygram11.config")
+      .attr("config")
+      .cast<py::dict>()[k]
       .cast<py::ssize_t>();
 }
 
@@ -1175,17 +1140,25 @@ py::array_t<py::ssize_t> f1d(py::array_t<Tx, py::array::c_style> x, py::ssize_t
   auto values = pg11::zeros<py::ssize_t>(nbins);
   pg11::faxis_t<double> ax{nbins, xmin, xmax};
   auto nx = x.shape(0);
-  if (nx < pg11::fwpt1d()) {  // serial
-    if (flow)
-      pg11::one::s_loop_incf(x.data(), nx, ax, values.mutable_data());
-    else
-      pg11::one::s_loop_excf(x.data(), nx, ax, values.mutable_data());
-  }
-  else {  // parallel
-    if (flow)
-      pg11::one::p_loop_incf(x.data(), nx, ax, values.mutable_data());
-    else
-      pg11::one::p_loop_excf(x.data(), nx, ax, values.mutable_data());
+  auto threshold = pg11::config_threshold("thresholds.fix1d");
+  {
+    py::gil_scoped_release release;
+    if (nx < threshold) {  // serial
+      if (flow) {
+        pg11::one::s_loop_incf(x.data(), nx, ax, values.mutable_data());
+      }
+      else {
+        pg11::one::s_loop_excf(x.data(), nx, ax, values.mutable_data());
+      }
+    }
+    else {  // parallel
+      if (flow) {
+        pg11::one::p_loop_incf(x.data(), nx, ax, values.mutable_data());
+      }
+      else {
+        pg11::one::p_loop_excf(x.data(), nx, ax, values.mutable_data());
+      }
+    }
   }
   return values;
 }
@@ -1197,23 +1170,30 @@ py::tuple f1dw(py::array_t<Tx, py::array::c_style> x, py::array_t<Tw, py::array:
   auto variances = pg11::zeros<Tw>(nbins);
   auto nx = x.shape(0);
   pg11::faxis_t<double> ax{nbins, xmin, xmax};
-  if (nx < pg11::fwpt1d()) {  // serial
-    if (flow)
-      pg11::one::s_loop_incf(x.data(), w.data(), nx, ax, values.mutable_data(),
-                             variances.mutable_data());
-    else
-      pg11::one::s_loop_excf(x.data(), w.data(), nx, ax, values.mutable_data(),
-                             variances.mutable_data());
+  auto threshold = pg11::config_threshold("thresholds.fix1d");
+  {
+    py::gil_scoped_release release;
+    if (nx < threshold) {  // serial
+      if (flow) {
+        pg11::one::s_loop_incf(x.data(), w.data(), nx, ax, values.mutable_data(),
+                               variances.mutable_data());
+      }
+      else {
+        pg11::one::s_loop_excf(x.data(), w.data(), nx, ax, values.mutable_data(),
+                               variances.mutable_data());
+      }
+    }
+    else {  // parallel
+      if (flow) {
+        pg11::one::p_loop_incf(x.data(), w.data(), nx, ax, values.mutable_data(),
+                               variances.mutable_data());
+      }
+      else {
+        pg11::one::p_loop_excf(x.data(), w.data(), nx, ax, values.mutable_data(),
+                               variances.mutable_data());
+      }
+    }
   }
-  else {  // parallel
-    if (flow)
-      pg11::one::p_loop_incf(x.data(), w.data(), nx, ax, values.mutable_data(),
-                             variances.mutable_data());
-    else
-      pg11::one::p_loop_excf(x.data(), w.data(), nx, ax, values.mutable_data(),
-                             variances.mutable_data());
-  }
-  pg11::arr_sqrt(variances.mutable_data(), nbins);
   return py::make_tuple(values, variances);
 }
 
@@ -1223,19 +1203,24 @@ py::tuple f1dmw(py::array_t<Tx> x, py::array_t<Tw> w, py::ssize_t nbins, double
   auto values = pg11::zeros<Tw>(nbins, w.shape(1));
   auto variances = pg11::zeros<Tw>(nbins, w.shape(1));
   pg11::faxis_t<double> ax{nbins, xmin, xmax};
-  if (x.shape(0) < pg11::fwmwpt1d()) {  // serial
-    if (flow)
-      pg11::one::s_loop_incf(x, w, ax, values, variances);
-    else
-      pg11::one::s_loop_excf(x, w, ax, values, variances);
+  auto threshold = pg11::config_threshold("thresholds.fix1dmw");
+  {
+    py::gil_scoped_release release;
+    if (x.shape(0) < threshold) {  // serial
+      if (flow) {
+        pg11::one::s_loop_incf(x, w, ax, values, variances);
+      }
+      else {
+        pg11::one::s_loop_excf(x, w, ax, values, variances);
+      }
+    }
+    else {  // parallel
+      if (flow)
+        pg11::one::p_loop_incf(x, w, ax, values, variances);
+      else
+        pg11::one::p_loop_excf(x, w, ax, values, variances);
+    }
   }
-  else {  // parallel
-    if (flow)
-      pg11::one::p_loop_incf(x, w, ax, values, variances);
-    else
-      pg11::one::p_loop_excf(x, w, ax, values, variances);
-  }
-  pg11::arr_sqrt(variances);
   return py::make_tuple(values, variances);
 }
 
@@ -1246,17 +1231,25 @@ py::array_t<py::ssize_t> v1d(py::array_t<Tx, py::array::c_style> x,
   std::vector<double> edges_v(edges.data(), edges.data() + nedges);
   auto values = pg11::zeros<py::ssize_t>(nedges - 1);
   auto nx = x.shape(0);
-  if (nx < pg11::vwpt1d()) {  // serial
-    if (flow)
-      pg11::one::s_loop_incf(x.data(), nx, edges_v, values.mutable_data());
-    else
-      pg11::one::s_loop_excf(x.data(), nx, edges_v, values.mutable_data());
-  }
-  else {  // parallel
-    if (flow)
-      pg11::one::p_loop_incf(x.data(), nx, edges_v, values.mutable_data());
-    else
-      pg11::one::p_loop_excf(x.data(), nx, edges_v, values.mutable_data());
+  auto threshold = pg11::config_threshold("thresholds.var1d");
+  {
+    py::gil_scoped_release release;
+    if (nx < threshold) {  // serial
+      if (flow) {
+        pg11::one::s_loop_incf(x.data(), nx, edges_v, values.mutable_data());
+      }
+      else {
+        pg11::one::s_loop_excf(x.data(), nx, edges_v, values.mutable_data());
+      }
+    }
+    else {  // parallel
+      if (flow) {
+        pg11::one::p_loop_incf(x.data(), nx, edges_v, values.mutable_data());
+      }
+      else {
+        pg11::one::p_loop_excf(x.data(), nx, edges_v, values.mutable_data());
+      }
+    }
   }
   return values;
 }
@@ -1270,23 +1263,30 @@ py::tuple v1dw(py::array_t<Tx, py::array::c_style> x, py::array_t<Tw, py::array:
   auto values = pg11::zeros<Tw>(nbins);
   auto variances = pg11::zeros<Tw>(nbins);
   auto nx = x.shape(0);
-  if (nx < pg11::vwpt1d()) {  // serial
-    if (flow)
-      pg11::one::s_loop_incf(x.data(), w.data(), nx, edges_v, values.mutable_data(),
-                             variances.mutable_data());
-    else
-      pg11::one::s_loop_excf(x.data(), w.data(), nx, edges_v, values.mutable_data(),
-                             variances.mutable_data());
+  auto threshold = pg11::config_threshold("thresholds.var1d");
+  {
+    py::gil_scoped_release release;
+    if (nx < threshold) {  // serial
+      if (flow) {
+        pg11::one::s_loop_incf(x.data(), w.data(), nx, edges_v, values.mutable_data(),
+                               variances.mutable_data());
+      }
+      else {
+        pg11::one::s_loop_excf(x.data(), w.data(), nx, edges_v, values.mutable_data(),
+                               variances.mutable_data());
+      }
+    }
+    else {  // parallel
+      if (flow) {
+        pg11::one::p_loop_incf(x.data(), w.data(), nx, edges_v, values.mutable_data(),
+                               variances.mutable_data());
+      }
+      else {
+        pg11::one::p_loop_excf(x.data(), w.data(), nx, edges_v, values.mutable_data(),
+                               variances.mutable_data());
+      }
+    }
   }
-  else {  // parallel
-    if (flow)
-      pg11::one::p_loop_incf(x.data(), w.data(), nx, edges_v, values.mutable_data(),
-                             variances.mutable_data());
-    else
-      pg11::one::p_loop_excf(x.data(), w.data(), nx, edges_v, values.mutable_data(),
-                             variances.mutable_data());
-  }
-  pg11::arr_sqrt(variances.mutable_data(), nbins);
   return py::make_tuple(values, variances);
 }
 
@@ -1298,19 +1298,24 @@ py::tuple v1dmw(py::array_t<Tx> x, py::array_t<Tw> w, py::array_t<double> edges,
   std::vector<double> edges_v(edges.data(), edges.data() + nedges);
   auto values = pg11::zeros<Tw>(nbins, w.shape(1));
   auto variances = pg11::zeros<Tw>(nbins, w.shape(1));
-  if (x.shape(0) < pg11::vwmwpt1d()) {  // serial
-    if (flow)
-      pg11::one::s_loop_incf(x, w, edges_v, values, variances);
-    else
-      pg11::one::s_loop_excf(x, w, edges_v, values, variances);
+  auto threshold = pg11::config_threshold("thresholds.var1dmw");
+  {
+    py::gil_scoped_release release;
+    if (x.shape(0) < threshold) {  // serial
+      if (flow) {
+        pg11::one::s_loop_incf(x, w, edges_v, values, variances);
+      }
+      else {
+        pg11::one::s_loop_excf(x, w, edges_v, values, variances);
+      }
+    }
+    else {  // parallel
+      if (flow)
+        pg11::one::p_loop_incf(x, w, edges_v, values, variances);
+      else
+        pg11::one::p_loop_excf(x, w, edges_v, values, variances);
+    }
   }
-  else {  // parallel
-    if (flow)
-      pg11::one::p_loop_incf(x, w, edges_v, values, variances);
-    else
-      pg11::one::p_loop_excf(x, w, edges_v, values, variances);
-  }
-  pg11::arr_sqrt(variances);
   return py::make_tuple(values, variances);
 }
 
@@ -1321,17 +1326,25 @@ py::array_t<py::ssize_t> f2d(py::array_t<Tx> x, py::array_t<Ty> y, py::ssize_t n
   auto values = pg11::zeros<py::ssize_t>(nbinsx, nbinsy);
   pg11::faxis_t<double> axx{nbinsx, xmin, xmax};
   pg11::faxis_t<double> axy{nbinsy, ymin, ymax};
-  if (x.shape(0) < pg11::fwpt2d()) {  // serial
-    if (flow)
-      pg11::two::s_loop_incf(x.data(), y.data(), x.shape(0), axx, axy, values);
-    else
-      pg11::two::s_loop_excf(x.data(), y.data(), x.shape(0), axx, axy, values);
-  }
-  else {
-    if (flow)
-      pg11::two::p_loop_incf(x.data(), y.data(), x.shape(0), axx, axy, values);
-    else
-      pg11::two::p_loop_excf(x.data(), y.data(), x.shape(0), axx, axy, values);
+  auto threshold = pg11::config_threshold("thresholds.fix2d");
+  {
+    py::gil_scoped_release release;
+    if (x.shape(0) < threshold) {  // serial
+      if (flow) {
+        pg11::two::s_loop_incf(x.data(), y.data(), x.shape(0), axx, axy, values);
+      }
+      else {
+        pg11::two::s_loop_excf(x.data(), y.data(), x.shape(0), axx, axy, values);
+      }
+    }
+    else {
+      if (flow) {
+        pg11::two::p_loop_incf(x.data(), y.data(), x.shape(0), axx, axy, values);
+      }
+      else {
+        pg11::two::p_loop_excf(x.data(), y.data(), x.shape(0), axx, axy, values);
+      }
+    }
   }
   return values;
 }
@@ -1344,23 +1357,30 @@ py::tuple f2dw(py::array_t<Tx> x, py::array_t<Ty> y, py::array_t<Tw> w, py::ssiz
   auto variances = pg11::zeros<Tw>(nbinsx, nbinsy);
   pg11::faxis_t<double> axx{nbinsx, xmin, xmax};
   pg11::faxis_t<double> axy{nbinsy, ymin, ymax};
-  if (x.shape(0) < pg11::fwpt2d()) {  // serial
-    if (flow)
-      pg11::two::s_loop_incf(x.data(), y.data(), w.data(), x.shape(0), axx, axy, values,
-                             variances);
-    else
-      pg11::two::s_loop_excf(x.data(), y.data(), w.data(), x.shape(0), axx, axy, values,
-                             variances);
+  auto threshold = pg11::config_threshold("thresholds.fix2d");
+  {
+    py::gil_scoped_release release;
+    if (x.shape(0) < threshold) {  // serial
+      if (flow) {
+        pg11::two::s_loop_incf(x.data(), y.data(), w.data(), x.shape(0), axx, axy, values,
+                               variances);
+      }
+      else {
+        pg11::two::s_loop_excf(x.data(), y.data(), w.data(), x.shape(0), axx, axy, values,
+                               variances);
+      }
+    }
+    else {
+      if (flow) {
+        pg11::two::p_loop_incf(x.data(), y.data(), w.data(), x.shape(0), axx, axy, values,
+                               variances);
+      }
+      else {
+        pg11::two::p_loop_excf(x.data(), y.data(), w.data(), x.shape(0), axx, axy, values,
+                               variances);
+      }
+    }
   }
-  else {
-    if (flow)
-      pg11::two::p_loop_incf(x.data(), y.data(), w.data(), x.shape(0), axx, axy, values,
-                             variances);
-    else
-      pg11::two::p_loop_excf(x.data(), y.data(), w.data(), x.shape(0), axx, axy, values,
-                             variances);
-  }
-  pg11::arr_sqrt(variances);
   return py::make_tuple(values, variances);
 }
 
@@ -1375,17 +1395,25 @@ py::array_t<py::ssize_t> v2d(py::array_t<Tx> x, py::array_t<Ty> y,
   auto values = pg11::zeros<py::ssize_t>(nbinsx, nbinsy);
   std::vector<double> edgesx_v(xbins.data(), xbins.data() + nedgesx);
   std::vector<double> edgesy_v(ybins.data(), ybins.data() + nedgesy);
-  if (x.shape(0) < pg11::vwpt2d()) {
-    if (flow)
-      pg11::two::s_loop_incf(x.data(), y.data(), x.shape(0), edgesx_v, edgesy_v, values);
-    else
-      pg11::two::s_loop_excf(x.data(), y.data(), x.shape(0), edgesx_v, edgesy_v, values);
-  }
-  else {
-    if (flow)
-      pg11::two::p_loop_incf(x.data(), y.data(), x.shape(0), edgesx_v, edgesy_v, values);
-    else
-      pg11::two::p_loop_excf(x.data(), y.data(), x.shape(0), edgesx_v, edgesy_v, values);
+  auto threshold = pg11::config_threshold("thresholds.var2d");
+  {
+    py::gil_scoped_release release;
+    if (x.shape(0) < threshold) {
+      if (flow) {
+        pg11::two::s_loop_incf(x.data(), y.data(), x.shape(0), edgesx_v, edgesy_v, values);
+      }
+      else {
+        pg11::two::s_loop_excf(x.data(), y.data(), x.shape(0), edgesx_v, edgesy_v, values);
+      }
+    }
+    else {
+      if (flow) {
+        pg11::two::p_loop_incf(x.data(), y.data(), x.shape(0), edgesx_v, edgesy_v, values);
+      }
+      else {
+        pg11::two::p_loop_excf(x.data(), y.data(), x.shape(0), edgesx_v, edgesy_v, values);
+      }
+    }
   }
   return values;
 }
@@ -1401,23 +1429,30 @@ py::tuple v2dw(py::array_t<Tx> x, py::array_t<Ty> y, py::array_t<Tw> w,
   auto variances = pg11::zeros<Tw>(nbinsx, nbinsy);
   std::vector<double> edgesx_v(xbins.data(), xbins.data() + nedgesx);
   std::vector<double> edgesy_v(ybins.data(), ybins.data() + nedgesy);
-  if (x.shape(0) < pg11::vwpt2d()) {
-    if (flow)
-      pg11::two::s_loop_incf(x.data(), y.data(), w.data(), x.shape(0), edgesx_v, edgesy_v,
-                             values, variances);
-    else
-      pg11::two::s_loop_excf(x.data(), y.data(), w.data(), x.shape(0), edgesx_v, edgesy_v,
-                             values, variances);
+  auto threshold = pg11::config_threshold("thresholds.var2d");
+  {
+    py::gil_scoped_release release;
+    if (x.shape(0) < threshold) {
+      if (flow) {
+        pg11::two::s_loop_incf(x.data(), y.data(), w.data(), x.shape(0), edgesx_v, edgesy_v,
+                               values, variances);
+      }
+      else {
+        pg11::two::s_loop_excf(x.data(), y.data(), w.data(), x.shape(0), edgesx_v, edgesy_v,
+                               values, variances);
+      }
+    }
+    else {
+      if (flow) {
+        pg11::two::p_loop_incf(x.data(), y.data(), w.data(), x.shape(0), edgesx_v, edgesy_v,
+                               values, variances);
+      }
+      else {
+        pg11::two::p_loop_excf(x.data(), y.data(), w.data(), x.shape(0), edgesx_v, edgesy_v,
+                               values, variances);
+      }
+    }
   }
-  else {
-    if (flow)
-      pg11::two::p_loop_incf(x.data(), y.data(), w.data(), x.shape(0), edgesx_v, edgesy_v,
-                             values, variances);
-    else
-      pg11::two::p_loop_excf(x.data(), y.data(), w.data(), x.shape(0), edgesx_v, edgesy_v,
-                             values, variances);
-  }
-  pg11::arr_sqrt(variances);
   return py::make_tuple(values, variances);
 }
 
@@ -1492,6 +1527,9 @@ PYBIND11_MODULE(_backend, m) {
   m.doc() = "pygram11 C++ backend.";
   m.def("_omp_get_max_threads", []() { return omp_get_max_threads(); });
 
+  m.def("_get_config_threshold",
+        [](const std::string& k) { return pg11::config_threshold(k.c_str()); });
+
   using boost::mp11::mp_for_each;
   mp_for_each<pg_types>([&](const auto& Ts) { inject1d(m, Ts); });
   mp_for_each<pg_types_and_weight>([&](const auto& Ts) { inject_1dw(m, Ts); });
